package RM;

import java.rmi.*;
import java.util.ArrayList;
import java.util.Hashtable;

import Resources.*;
import LockManager.*;


class Reservation
{
  public int customer;
  //an array list of ReservableElement for that reservation

  public ArrayList reservation_items;

  public Reservation(int customer,ReservableElement i, int price)
  {
    this.customer = customer;
    reservation_items = new ArrayList();
    reservation_items.add(i);
  }
}

/**
 * This is the class implementing the Resource Manager.
 * This class should implement the IResourceManager interface
 * for testing purposes in the beginning.
 * Later, the client would not interact with the RM directly, 
 * but through the Workflow Controller
 */
public class MyRM extends java.rmi.server.UnicastRemoteObject
	   	  implements IResourceManager
{

     /**
     * Two hashtables store the database.
     * This is the first functionality you would re-implement
     * by replacing it with the Storage class
     */

     // Resources: cars, flights, room and customers
     Hashtable items;

     // Reservations
     Hashtable reservations;
     
     public MyRM() throws java.rmi.RemoteException
     {
	 items		= new Hashtable();
	 reservations	= new Hashtable();
     }
    
     /**
     * Start new transaction
     *	@return a unique transaction id
     */
     public int start()
     {
	return 10;
     }
    
     public void commit(int context)
     {
      // transactionManager.Commit( context );
     }   

     /**
     * abort transaction context
     * @param context the transaction id
     */
     public void abort(int context)
     {
	
     }

     /**
     * Transaction context is adding resource i with the given 
     * amount and price.
     * See the interface for full description of the method
     */
     public boolean add(int context,ReservableElement i, int amount, int price)
     {
	ReservableElement ii = (ReservableElement)items.get(i.getKey());
	
	if (ii!=null)
	{
	    ii.addAvailable(amount,price);
	}
	else 
	   {
	     ReservableElement item=new ReservableElement();
	     i.copy(item);
	     item.addAvailable(amount,price);
	     items.put(i.getKey(),item);
	   }
	
	return true;
      }

   
     public boolean delete(int context, ReservableElement i)
     {
	if(items.remove(i.getName())==null) return false;

	return true;
     } 

     public void shutdown(int diskWritesToWait)
     {
     }

     public void SelfDestruct(int diskWritesToWait)
     {
     }

     public int queryPrice(int context, ReservableElement i)
     {
	 return ((ReservableElement)items.get(i.getName())).getPrice();
     }

     public int query(int context, ReservableElement i)
     {
	 ReservableElement ii = (ReservableElement)items.get(i.getName());

	 if( ii == null )
	    return 0;
	 else
	    return ii.getAvailable();
     }

         
     public int newCustomer(int Xid)
	throws RemoteException
//, InvalidTransactionException, TransactionAbortedException
     {
	return 0;
     }
     
     
     public int queryCustomer(int context,int customer) 
     {
	 return 0;
     }

     public String queryCustomerInfo(int context, int customer)
	 throws InvalidTransactionException
     {

      String order;
      order = "";

      if (reservations.get(new Integer(customer))==null)
      {
	  return order;
      }

      Reservation r 	  = (Reservation) reservations.get(new Integer(customer));
      ArrayList reserved  = r.reservation_items;

      for (int i=0; i<reserved.size(); i++)
      {
	  if(order.length() > 0)
	  {
	      order += ",";
	  }
	 
	  order += ((ReservableElement)reserved.get(i)).getName();
      }

      return order;
    }
	
    /**
    * Reserve an item for a customer.
    *
    * @param context Transaction ID (obtained via start)
    * @param customer ID of customer for reservation
    * @param location Name of location for reservation
    * @return success
    * @throws RemoteException RMI failure
    * @throws TransactionAbortedException Transaction was previously aborted
    * @throws InvalidTransactionException Invalid xid
    * @see #addCars
    * @see #deleteCars
    * @see #queryCars
    * @see #queryCarsPrice
    */
    public boolean reserve(int context, int customer, ReservableElement i) throws RemoteException,
	       TransactionAbortedException, InvalidTransactionException
    {
	 ReservableElement e = (ReservableElement)items.get(i.getName());

	 if(e == null)
	 {
	     throw new InvalidTransactionException(context,"Item does not exists"+ i.getName());
	 }

	 int amt = i.getAvailable()<=0?1:i.getAvailable();

	 if(e.getAvailable() > amt)
	 {
	    Reservation r = (Reservation)reservations.get(new Integer(customer));

	    if(r == null) 
	    {
		r = new Reservation(customer, i, e.getPrice());
		reservations.put(new Integer(customer), r);
	    }
	    else
		{
		   r.reservation_items.add(i);
		}

	    e.reduceAvailable(amt);

	    return true;
	  }
          else
             {
	       throw new InvalidTransactionException(context, "No more items available");
	     }
    
      }

      public void dropReservations(int context, int customer)
      {
	 Reservation r = (Reservation) reservations.get(new Integer(customer));
	 ArrayList reserved=r.reservation_items;	

	 for (int i=0;i<reserved.size();i++)
	 {
	     ((ReservableElement)items.get(reserved.get(i))).addAvailable(1);
	 }
	 reservations.remove(new Integer(customer));
	     
      }

      public boolean shutdown()
      {
	return true;
      }

      public boolean selfDestruct(int diskWritesToWait) 
      {
	 return true;
      }
     

      public boolean reserveItinerary(int customer, int[] flights, String location, boolean bCar, boolean bRoom)
		     throws RemoteException, TransactionAbortedException, InvalidTransactionException
      {
	 return false;
      }


      public boolean reserveRoom(int context, int customer, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }

      public boolean reserveCar(int context, int customer, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }

      public boolean addFlight(int context, int flight, int flightSeats, int flightPrice)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }

      public  boolean deleteFlight(int context, int flight)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }


      public  boolean addCars(int context, String location, int numCars, int price)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }


      public  boolean deleteCars(int context, String location, int numCars)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }


      public  boolean addRooms(int context, String location, int numRooms, int price)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }


      public  boolean deleteRooms(int context, String location, int numRooms)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }


      public  int queryFlight(int context, int flight)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 0;
      }


      public  int queryFlightPrice(int context, int flight)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 0;
      }


      public  int queryRooms(int context, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 0;
      }


      public int queryRoomsPrice(int context, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 0;
      }

      public  int queryCars(int context, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 4;
      }


      public int queryCarsPrice(int context, String location)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return 5;
      }


      public boolean deleteCustomer(int context, int customer)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }

      public boolean reserveFlight(int context, int customer, int flight)
	throws RemoteException, 
	       TransactionAbortedException,
	       InvalidTransactionException
      {
	 return false;
      }



      /** 
      * This is a separate process: wait for clients to connect
      */
      public static void main(String args[]) 
      {
        System.setSecurityManager(new RMISecurityManager());
	String name ="";	

        if (args.length != 1) 
	{
            System.err.println("Usage: RM.MyRM <name>");
	    name="testRM";
        }
        else 
           {
	      name= args[0];
	   }
	
        // Bind the RM and begin processing requests.
        try 
	{
            IResourceManager obj = new MyRM();
            Naming.rebind(name, obj);
	}
        catch (Exception e) 
	{
            System.err.println(name + " not bound:" + e);
        }
    }
}
